@@ -1,11 +1,18 @@
 from logging import getLogger
-from typing import Tuple, Any, Dict, Union
+from typing import Any
+from typing import Dict
+from typing import Tuple
+from typing import Union
 
 import keyring
-from jira import JIRA, JIRAError
-from prompt_toolkit.shortcuts import input_dialog, yes_no_dialog
+from jira import JIRA
+from jira import JIRAError
+from prompt_toolkit.shortcuts import input_dialog
+from prompt_toolkit.shortcuts import yes_no_dialog
 from urllib3 import disable_warnings
 
+from jira_select.types import InstanceDefinition
+
 from ..constants import APP_NAME
 from ..exceptions import UserError
 from ..plugin import BaseCommand
@@ -69,7 +76,9 @@ class Command(BaseCommand):
 
             try:
                 JIRA(
-                    options, basic_auth=(username, password), max_retries=0,
+                    options,
+                    basic_auth=(username, password),
+                    max_retries=0,
                 )
                 break
             except JIRAError:
@@ -83,11 +92,11 @@ class Command(BaseCommand):
                 if not result:
                     raise UserError("Aborted; configuration not saved.")
 
-        self.config.setdefault("instances", {})[self.options.instance_name] = {
-            "url": instance_url,
-            "username": username,
-            "verify": verify,
-        }
+        self.config.instances[self.options.instance_name] = InstanceDefinition(
+            url=instance_url,
+            username=username,
+            verify=verify,
+        )
         self.save_config()
 
         store_password = yes_no_dialog(
