@@ -1,5 +1,8 @@
 import csv
-from typing import Any, Dict, List
+from io import TextIOWrapper
+from typing import Any
+from typing import Dict
+from typing import List
 
 from ..plugin import BaseFormatter
 
@@ -13,14 +16,22 @@ class Formatter(BaseFormatter):
         fields = []
 
         for field in self.executor.query.select:
-            fields.append(field["column"])
+            fields.append(field.column)
 
         return fields
 
     def open(self):
         super().open()
-        self.out = csv.DictWriter(self.stream, fieldnames=self._generate_fieldnames())
+        self._wrapped_stream = TextIOWrapper(
+            self.stream, encoding="utf-8", write_through=True, newline=""
+        )
+        self.out = csv.DictWriter(
+            self._wrapped_stream, fieldnames=self._generate_fieldnames()
+        )
         self.out.writeheader()
 
+    def close(self):
+        self._wrapped_stream.detach()
+
     def writerow(self, row: Dict[str, Any]):
         self.out.writerow(row)
