@@ -1,4 +1,6 @@
-from typing import Any, Dict, Iterable
+from typing import Any
+from typing import Dict
+from typing import Iterable
 
 from QueryableList import QueryableListMixed
 
@@ -6,7 +8,7 @@ from ..plugin import BaseFunction
 
 
 class Function(BaseFunction):
-    """ Returns entries in an iterable matching a provided django-style query.
+    """Returns entries in an iterable matching a provided django-style query.
 
     Returned rows are required to match at least all of the provided query
     parameters.
