@@ -1,43 +1,48 @@
 from __future__ import annotations
 
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
+from abc import abstractmethod
 from functools import total_ordering
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Iterable,
-    Iterator,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import Generator
+from typing import Iterable
+from typing import Iterator
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import Type
+from typing import Union
+from typing import cast
 
-from jira import JIRA, Issue
-from rich.progress import Progress, TaskID, BarColumn, TimeRemainingColumn
+from dotmap import DotMap
+from jira import JIRA
+from jira import Issue
+from rich.progress import BarColumn
+from rich.progress import Progress
+from rich.progress import TaskID
+from rich.progress import TimeRemainingColumn
 
 from .cache import MinimumRecencyCache
-from .plugin import BaseSource, get_installed_functions, get_installed_sources
-from .types import (
-    ExpressionList,
-    JqlList,
-    QueryDefinition,
-    SelectFieldDefinition,
-    Expression,
-    WhereParamDict,
-)
-from .utils import (
-    calculate_result_hash,
-    expression_includes_group_by,
-    get_cache_path,
-    get_field_data,
-    get_row_dict,
-    parse_sort_by_definition,
-    parse_select_definition,
-)
+from .exceptions import ExpressionParameterMissing
+from .plugin import BaseSource
+from .plugin import get_installed_functions
+from .plugin import get_installed_sources
+from .types import Expression
+from .types import ExpressionList
+from .types import JqlList
+from .types import QueryDefinition
+from .types import SelectFieldDefinition
+from .types import WhereParamDict
+from .utils import calculate_result_hash
+from .utils import expression_includes_group_by
+from .utils import find_missing_parameters
+from .utils import get_cache_path
+from .utils import get_field_data
+from .utils import get_row_dict
+from .utils import parse_select_definition
+from .utils import parse_sort_by_definition
 
 
 @total_ordering
@@ -87,9 +92,22 @@ class Result(metaclass=ABCMeta):
         self,
         expression: Expression,
         group_by: Optional[ExpressionList] = None,
-        field_name_map: Optional[Dict[str, str]] = None,
+        field_name_map: Optional[Dict[str, Any]] = None,
         functions: Optional[Dict[str, Callable]] = None,
     ):
+        params: Dict[str, str] = cast(
+            Dict[str, str],
+            field_name_map.get("params", {}) if field_name_map is not None else {},
+        )
+        if missing := find_missing_parameters(
+            expression,
+            list(params.keys()),
+        ):
+            raise ExpressionParameterMissing(
+                "Parameter {params.%s} found in expression, but no parameter was specified!"
+                % missing[0]
+            )
+
         if group_by is None:
             group_by = []
 
@@ -101,7 +119,10 @@ class Result(metaclass=ABCMeta):
             row_source = self.single()
 
         return get_field_data(
-            row_source, expression, functions=functions, interpolations=field_name_map,
+            row_source,
+            expression,
+            functions=functions,
+            interpolations=field_name_map,
         )
 
 
@@ -130,7 +151,8 @@ class GroupedFieldContainer(list):
 
 class GroupedResult(Result):
     def __init__(
-        self, rows: List[SingleResult] = None,
+        self,
+        rows: List[SingleResult] = None,
     ):
         super().__init__()
         self._rows: List[SingleResult] = rows if rows is not None else []
@@ -194,7 +216,7 @@ class Query:
     def select(self) -> List[SelectFieldDefinition]:
         fields: List[SelectFieldDefinition] = []
 
-        for field in self._definition["select"]:
+        for field in self._definition.select:
             if field == "*":
                 fields.extend(self._get_all_fields())
             else:
@@ -204,53 +226,53 @@ class Query:
 
     @property
     def from_(self) -> str:
-        return self._definition["from"]
+        return self._definition.from_
 
     @property
     def where(self) -> Union[JqlList, WhereParamDict]:
-        where = self._definition.get("where", [])
+        where = self._definition.where
         if isinstance(where, list):
             return self._ensure_str(where)
         return where
 
     @property
     def order_by(self) -> JqlList:
-        return self._ensure_str(self._definition.get("order_by", []))
+        return self._ensure_str(self._definition.order_by)
 
     @property
     def filter(self) -> ExpressionList:
-        return self._ensure_str(self._definition.get("filter", []))
+        return self._ensure_str(self._definition.filter_)
 
     @property
     def having(self) -> ExpressionList:
-        return self._ensure_str(self._definition.get("having", []))
+        return self._ensure_str(self._definition.having)
 
     @property
     def group_by(self) -> ExpressionList:
-        return self._ensure_str(self._definition.get("group_by", []))
+        return self._ensure_str(self._definition.group_by)
 
     @property
     def sort_by(self) -> List[Tuple[Expression, bool]]:
         return [
             parse_sort_by_definition(definition)
-            for definition in self._ensure_str(self._definition.get("sort_by", []))
+            for definition in self._ensure_str(self._definition.sort_by)
         ]
 
     @property
     def expand(self) -> List[str]:
-        return self._ensure_str(self._definition.get("expand", []))
+        return self._ensure_str(self._definition.expand)
 
     @property
     def limit(self) -> Optional[int]:
-        return self._definition.get("limit")
+        return self._definition.limit
 
     @property
     def cap(self) -> Optional[int]:
-        return self._definition.get("cap")
+        return self._definition.cap
 
     @property
     def cache(self) -> Optional[Tuple[Optional[int], Optional[int]]]:
-        value = self._definition.get("cache")
+        value = self._definition.cache
         if isinstance(value, int):
             return (
                 value,
@@ -279,29 +301,9 @@ class NullProgressbar:
         pass
 
 
-class NullCache:
-    def __init__(self, *args, **kwargs):
-        pass
-
-    def __getitem__(self, key):
-        raise KeyError(key)
-
-    def get(self, key, default=None, **kwargs) -> Optional[Any]:
-        return default
-
-    def touch(self, key, **kwargs):
-        return
-
-    def set(self, key, value, **kwargs):
-        return
-
-    def add(self, key, value, **kwargs):
-        return
-
-
 class CounterChannel:
     def __init__(self):
-        self._counter: int = 2 ** 32
+        self._counter: int = 2**32
 
     def zero(self):
         self._counter = 0
@@ -328,6 +330,7 @@ class Executor:
         definition: QueryDefinition,
         enable_cache: bool = True,
         progress_bar: bool = False,
+        parameters: Optional[Dict[str, str]] = None,
     ):
         self._query: Query = Query(jira, definition)
         # self._definition: QueryDefinition = clean_query_definition(definition)
@@ -335,10 +338,10 @@ class Executor:
         self._functions: Dict[str, Callable] = get_installed_functions(jira)
         self._progress_bar_enabled = progress_bar
 
-        self._cache: Union[MinimumRecencyCache, NullCache] = NullCache()
-        if enable_cache:
-            self._cache = MinimumRecencyCache(get_cache_path())
+        self._cache = MinimumRecencyCache(get_cache_path())
 
+        self._enable_cache = enable_cache
+        self._parameters: Dict[str, str] = parameters or {}
         self._field_name_map: Dict[str, str] = FieldNameMap()
 
     @property
@@ -346,7 +349,7 @@ class Executor:
         return self._jira
 
     @property
-    def cache(self) -> Union[MinimumRecencyCache, NullCache]:
+    def cache(self) -> MinimumRecencyCache:
         return self._cache
 
     @property
@@ -358,15 +361,26 @@ class Executor:
         return self._functions
 
     @property
-    def field_name_map(self) -> Dict[str, str]:
+    def field_name_map(self) -> Dict[str, Any]:
         if not self._field_name_map:
             for jira_field in self.jira.fields():
                 self._field_name_map[jira_field["name"]] = jira_field["id"]
 
+            self._field_name_map["params"] = DotMap(self._parameters)
+
         return self._field_name_map
 
     def _get_jql(self) -> str:
         query = " AND ".join(f"({q})" for q in self.query.where)
+
+        if missing := find_missing_parameters(query, list(self._parameters.keys())):
+            raise ExpressionParameterMissing(
+                "Parameter {params.%s} found in expression, but no parameter was specified!"
+                % missing[0]
+            )
+
+        query = query.format(params=DotMap(self._parameters))
+
         order_by_fields = ", ".join(self.query.order_by)
 
         if order_by_fields:
@@ -386,7 +400,7 @@ class Executor:
             ]
         )
 
-        if self.query.cache:
+        if self.query.cache and self._enable_cache:
             try:
                 min_recency, _ = self.query.cache
                 if min_recency is None:
@@ -452,7 +466,11 @@ class Executor:
         for row in iterator:
             self.progress.update(task, total=input_channel.get(), visible=True)
 
-            row_hash = calculate_result_hash(row, self.query.group_by, self.functions,)
+            row_hash = calculate_result_hash(
+                row,
+                self.query.group_by,
+                self.functions,
+            )
             if row_hash not in groups:
                 output_channel.increment()
                 groups[row_hash] = GroupedResult()
@@ -517,8 +535,9 @@ class Executor:
         result: Dict[str, Any] = {}
 
         for field_defn in self.query.select:
-            result[field_defn["column"]] = self.evaluate_expression(
-                row, field_defn["expression"],
+            result[field_defn.column] = self.evaluate_expression(
+                row,
+                field_defn.expression,
             )
 
         return result
@@ -538,7 +557,12 @@ class Executor:
         phases: List[
             Tuple[
                 Callable[
-                    [Iterator[Result], TaskID, CounterChannel, CounterChannel,],
+                    [
+                        Iterator[Result],
+                        TaskID,
+                        CounterChannel,
+                        CounterChannel,
+                    ],
                     Iterator[Result],
                 ],
                 TaskID,
@@ -546,16 +570,36 @@ class Executor:
         ] = []
         if self.query.filter:
             filter_task = self.progress.add_task("filter", total=0, visible=False)
-            phases.append((self._process_filter, filter_task,),)
+            phases.append(
+                (
+                    self._process_filter,
+                    filter_task,
+                ),
+            )
         if self.query.group_by:
             group_by_task = self.progress.add_task("group_by", total=0, visible=False)
-            phases.append((self._process_group_by, group_by_task,),)
+            phases.append(
+                (
+                    self._process_group_by,
+                    group_by_task,
+                ),
+            )
         if self.query.having:
             having_task = self.progress.add_task("having", total=0, visible=False)
-            phases.append((self._process_having, having_task,),)
+            phases.append(
+                (
+                    self._process_having,
+                    having_task,
+                ),
+            )
         if self.query.sort_by:
             sort_by_task = self.progress.add_task("sort_by", total=0, visible=False)
-            phases.append((self._process_sort_by, sort_by_task,),)
+            phases.append(
+                (
+                    self._process_sort_by,
+                    sort_by_task,
+                ),
+            )
         select_task = self.progress.add_task("select", total=0, visible=False)
 
         # Link up each generator with its source; these will vary
