@@ -44,6 +44,9 @@ class QueryDefinition(BaseModel):
     cap: Optional[int]
     cache: Optional[Union[int, Tuple[Optional[int], Optional[int]]]]
 
+    class Config:
+        allow_population_by_field_name = True
+
 
 class SchemaRow(BaseModel):
     id: str
