@@ -1,11 +1,14 @@
+import uuid
 from unittest.mock import Mock
 
+import pytest
 from dotmap import DotMap
 from jira import Issue
 from jira.resources import Resource
 
-from jira_select.types import QueryDefinition
+from jira_select.exceptions import ExpressionParameterMissing
 from jira_select.query import Executor
+from jira_select.types import QueryDefinition
 
 from .base import JiraSelectTestCase
 
@@ -95,81 +98,113 @@ class TestQuery(JiraSelectTestCase):
         )
 
     def test_simple(self):
-        query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+            }
+        )
 
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)
         expected_results = [
-            {"key": "ALPHA-1",},
-            {"key": "ALPHA-3",},
-            {"key": "ALPHA-2",},
+            {
+                "key": "ALPHA-1",
+            },
+            {
+                "key": "ALPHA-3",
+            },
+            {
+                "key": "ALPHA-2",
+            },
         ]
 
         assert expected_results == actual_results
 
     def test_sort_by(self):
-        query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-            "sort_by": ["story_points desc", "key"],
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+                "sort_by": ["story_points desc", "key"],
+            }
+        )
 
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)
         expected_results = [
-            {"key": "ALPHA-3",},
-            {"key": "ALPHA-1",},
-            {"key": "ALPHA-2",},
+            {
+                "key": "ALPHA-3",
+            },
+            {
+                "key": "ALPHA-1",
+            },
+            {
+                "key": "ALPHA-2",
+            },
         ]
 
         assert expected_results == actual_results
 
     def test_filter(self):
-        query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-            "filter": ["summary != 'My Ticket #2'"],
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+                "filter": ["summary != 'My Ticket #2'"],
+            }
+        )
 
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)
         expected_results = [
-            {"key": "ALPHA-1",},
-            {"key": "ALPHA-3",},
+            {
+                "key": "ALPHA-1",
+            },
+            {
+                "key": "ALPHA-3",
+            },
         ]
 
         assert expected_results == actual_results
 
     def test_group_by_aggregation(self):
-        query: QueryDefinition = {
-            "select": ["issuetype", "len(key)"],
-            "from": "issues",
-            "group_by": ["issuetype"],
-            "sort_by": ["len(key)"],
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["issuetype", "len(key)"],
+                "from": "issues",
+                "group_by": ["issuetype"],
+                "sort_by": ["len(key)"],
+            }
+        )
 
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)
         expected_results = [
-            {"issuetype": "Bug", "len(key)": 1,},
-            {"issuetype": "Issue", "len(key)": 2,},
+            {
+                "issuetype": "Bug",
+                "len(key)": 1,
+            },
+            {
+                "issuetype": "Issue",
+                "len(key)": 2,
+            },
         ]
 
         assert expected_results == actual_results
 
     def test_cap(self):
-        query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-            "cap": 1,
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+                "cap": 1,
+            }
+        )
 
         query = Executor(self.mock_jira, query)
 
@@ -177,27 +212,37 @@ class TestQuery(JiraSelectTestCase):
         assert len(actual_results) == 1
 
     def test_simple_wprogress(self):
-        query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+            }
+        )
 
         query = Executor(self.mock_jira, query, True)
 
         actual_results = list(query)
         expected_results = [
-            {"key": "ALPHA-1",},
-            {"key": "ALPHA-3",},
-            {"key": "ALPHA-2",},
+            {
+                "key": "ALPHA-1",
+            },
+            {
+                "key": "ALPHA-3",
+            },
+            {
+                "key": "ALPHA-2",
+            },
         ]
 
         assert expected_results == actual_results
 
     def test_field_name_map(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ["key"],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ["key"],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
             return_value=[
                 {"name": "Story Points", "id": "customfield10010"},
@@ -209,18 +254,23 @@ class TestQuery(JiraSelectTestCase):
         expected_result = {
             "Story Points": "customfield10010",
             "Sprint": "customfield10011",
+            "params": DotMap(),
         }
         actual_result = query.field_name_map
 
         assert expected_result == actual_result
 
     def test_interpolated_value_nonspecial(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ['{customfield10011} as "mn"'],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ['{customfield10011} as "mn"'],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
-            return_value=[{"name": "Maiden Name", "id": "customfield10011"},]
+            return_value=[
+                {"name": "Maiden Name", "id": "customfield10011"},
+            ]
         )
 
         query = Executor(self.mock_jira, arbitrary_query, True)
@@ -235,12 +285,16 @@ class TestQuery(JiraSelectTestCase):
         assert expected_results == actual_results
 
     def test_interpolated_value_nonstring(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ['{Jellybean Guess} as "jb"'],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ['{Jellybean Guess} as "jb"'],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
-            return_value=[{"name": "Jellybean Guess", "id": "customfield10010"},]
+            return_value=[
+                {"name": "Jellybean Guess", "id": "customfield10010"},
+            ]
         )
 
         query = Executor(self.mock_jira, arbitrary_query, True)
@@ -255,12 +309,16 @@ class TestQuery(JiraSelectTestCase):
         assert expected_results == actual_results
 
     def test_interpolated_value_string(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ['{Maiden Name} as "mn"'],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ['{Maiden Name} as "mn"'],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
-            return_value=[{"name": "Maiden Name", "id": "customfield10011"},]
+            return_value=[
+                {"name": "Maiden Name", "id": "customfield10011"},
+            ]
         )
 
         query = Executor(self.mock_jira, arbitrary_query, True)
@@ -275,12 +333,16 @@ class TestQuery(JiraSelectTestCase):
         assert expected_results == actual_results
 
     def test_interpolated_value_resource(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ['{Result Object}.ok as "ro"'],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ['{Result Object}.ok as "ro"'],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
-            return_value=[{"name": "Result Object", "id": "customfield10012"},]
+            return_value=[
+                {"name": "Result Object", "id": "customfield10012"},
+            ]
         )
 
         query = Executor(self.mock_jira, arbitrary_query, True)
@@ -295,12 +357,16 @@ class TestQuery(JiraSelectTestCase):
         assert expected_results == actual_results
 
     def test_interpolated_value_nonresource(self):
-        arbitrary_query: QueryDefinition = {
-            "select": ['{Beep Boop} as "bb"'],
-            "from": "issues",
-        }
+        arbitrary_query = QueryDefinition.parse_obj(
+            {
+                "select": ['{Beep Boop} as "bb"'],
+                "from": "issues",
+            }
+        )
         self.mock_jira.fields = Mock(
-            return_value=[{"name": "Beep Boop", "id": "customfield10013"},]
+            return_value=[
+                {"name": "Beep Boop", "id": "customfield10013"},
+            ]
         )
 
         query = Executor(self.mock_jira, arbitrary_query, True)
@@ -310,11 +376,13 @@ class TestQuery(JiraSelectTestCase):
             assert isinstance(result["bb"], NonResource)
 
     def test_autoextract_sum(self):
-        query: QueryDefinition = {
-            "select": ['sum(transactions.byCurrency.usd) as "value"'],
-            "from": "issues",
-            "group_by": ["True"],
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ['sum(transactions.byCurrency.usd) as "value"'],
+                "from": "issues",
+                "group_by": ["True"],
+            }
+        )
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)[0]["value"]
@@ -323,16 +391,83 @@ class TestQuery(JiraSelectTestCase):
         assert expected_results == actual_results
 
     def test_autoextract_internal_array(self):
-        query: QueryDefinition = {
-            "select": [
-                'sum(extract(flatten_list(worklogs.worklogs), "timespentSeconds")) as "value"'
-            ],
-            "from": "issues",
-            "group_by": ["True"],
-        }
+        query = QueryDefinition.parse_obj(
+            {
+                "select": [
+                    'sum(extract(flatten_list(worklogs.worklogs), "timespentSeconds")) as "value"'
+                ],
+                "from": "issues",
+                "group_by": ["True"],
+            }
+        )
         query = Executor(self.mock_jira, query)
 
         actual_results = list(query)[0]["value"]
         expected_results = 105
 
         assert expected_results == actual_results
+
+    def test_param_expression_specified(self):
+        arbitrary_value = str(uuid.uuid4())
+
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["'{params.ok}' as \"value\""],
+                "from": "issues",
+                "cap": 1,
+            }
+        )
+
+        query = Executor(self.mock_jira, query, parameters={"ok": arbitrary_value})
+        results = list(query)
+
+        assert results[0]["value"] == arbitrary_value
+
+    def test_param_expression_unspecified(self):
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["'{params.ok}'"],
+                "from": "issues",
+                "cap": 1,
+            }
+        )
+
+        query = Executor(self.mock_jira, query)
+
+        with pytest.raises(ExpressionParameterMissing):
+            list(query)
+
+    def test_param_jql_specified(self):
+        arbitrary_value = str(uuid.uuid4())
+
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["'{params.ok}' as \"value\""],
+                "from": "issues",
+                "where": [
+                    '"{params.ok}"',
+                ],
+                "cap": 1,
+            }
+        )
+
+        query = Executor(self.mock_jira, query, parameters={"ok": arbitrary_value})
+
+        assert arbitrary_value in query._get_jql()
+
+    def test_param_jql_unspecified(self):
+        query = QueryDefinition.parse_obj(
+            {
+                "select": ["'{params.ok}' as \"value\""],
+                "from": "issues",
+                "where": [
+                    '"{params.ok}"',
+                ],
+                "cap": 1,
+            }
+        )
+
+        query = Executor(self.mock_jira, query)
+
+        with pytest.raises(ExpressionParameterMissing):
+            query._get_jql()
